@@ -3,13 +3,13 @@ package Email::Send::SMTP::Gmail;
 use strict;
 use warnings;
 use vars qw($VERSION);
-use utf8;
 
-$VERSION='0.46';
+$VERSION='0.56';
 
-require Net::SMTP::TLS::ButMaintained;
-require Net::SMTP::SSL;
-require Net::SMTP;
+require Net::SMTPS;
+#require Net::SMTP::SSL;
+#require Net::SMTP;
+#require Net::SMTP::TLS::ButMaintained;
 use MIME::Base64;
 use File::Spec;
 use LWP::MediaTypes;
@@ -22,11 +22,24 @@ sub new{
   my $smtp='smtp.gmail.com'; # Default value
   my $port='default'; # Default value
   my $layer='tls'; # Default value
+  my $auth='LOGIN'; # Default
+  my $ssl_verify_mode=''; #Default - Warning SSL_VERIFY_NONE
+
   $smtp=$properties{'-smtp'} if defined $properties{'-smtp'};
   $port=$properties{'-port'} if defined $properties{'-port'};
   $layer=$properties{'-layer'} if defined $properties{'-layer'};
-  $self->_initsmtp($smtp,$port,$properties{'-login'},$properties{'-pass'},$layer,$properties{'-debug'});
-  $self->{from}=$properties{'-login'};
+  $auth=$properties{'-auth'} if defined $properties{'-auth'};
+  $ssl_verify_mode=$properties{'-ssl_verify_mode'} if defined $properties{'-ssl_verify_mode'};
+
+  my $connect=$self->_initsmtp($smtp,$port,$properties{'-login'},$properties{'-pass'},$layer,$auth,$properties{'-debug'},$ssl_verify_mode,$properties{'-ssl_verify_path'},$properties{'-$ssl_verify_ca'});
+  return $connect if($connect==-1);
+
+  if(defined $properties{'-from'}){
+    $self->{from}=$properties{'-from'};
+  }
+  else{
+    $self->{from}=$properties{'-login'};
+  }
   return $self;
 }
 
@@ -37,35 +50,38 @@ sub _initsmtp{
   my $login=shift;
   my $pass=shift;
   my $layer=shift;
+  my $auth=shift;
   my $debug=shift;
+  my $ssl_mode=shift;
+  my $ssl_path=shift;
+  my $ssl_ca=shift;
 
   # The module sets the SMTP google but could use another!
-  print "Connecting to $smtp using $layer\n" if $debug;
-  if($layer eq 'tls'){
-    $port=25 if ($port eq 'default');
-    if (not $self->{sender} = Net::SMTP::TLS::ButMaintained->new($smtp, Port => $port, User=> $login, Password=> $pass)){
-        die "Could not connect to SMTP server\n";
-    }
-  }
-  elsif($layer eq 'ssl') {  # SSL
-    $port=465 if ($port eq 'default');
-    if (not $self->{sender} = Net::SMTP::SSL->new($smtp, Port => $port, Debug=> $debug)){
-        die "Could not connect to SMTP server\n";
-    }
-    # Authenticate
-    $self->{sender}->auth($login,$pass) || die "Authentication (SMTP) failed\n";
-  }
-  elsif($layer=~/plain|none/) { # Plain
-    $port=25 if ($port eq 'default');
-    if (not $self->{sender} = Net::SMTP->new($smtp, Port => $port, Debug=> $debug)){
-         die "Could not connect to SMTP server\n";
-    }
-    # Authenticate
-    if($layer eq 'plain'){
-      $self->{sender}->auth($login,$pass) || die "Authentication (SMTP) failed\n";
-    }
+  print "Connecting to $smtp using $layer with $auth\n" if $debug;
+  # Set port if default
+  if($port eq 'default'){
+      if($layer eq 'ssl'){
+          $port=465;
+      }
+      else{
+          $port=25;
+      }
   }
 
+  # Set security layer from $layer
+  my $sec=undef;
+  if($layer eq 'tls'){$sec='starttls';}
+  elsif($layer eq 'ssl'){$sec='ssl';}
+  # Connect
+  if (not $self->{sender} = Net::SMTPS->new($smtp, Port =>$port, doSSL=>$sec, Debug=>$debug, SSL_verify_mode=>$ssl_mode, SSL_ca_file=>$ssl_ca,SSL_ca_path=>$ssl_path)){
+      die "Could not connect to SMTP server\n";
+  }
+  if($auth ne 'none'){
+     unless($self->{sender}->auth($login,$pass,$auth)){
+         print "Authentication (SMTP) failed\n";
+         return -1;
+     }
+  }
   return $self;
 }
 
@@ -80,6 +96,7 @@ sub _checkfiles
 # Checks that all the attachments exist
   my $self=shift;
   my $attachs=shift;
+
   my @attachments=split(/,/,$attachs);
 
   foreach my $attach(@attachments)
@@ -104,18 +121,38 @@ sub _checkfiles
   return 1;
 }
 
-sub _createboundry
+sub _checkfilelist
 {
-# Create arbitrary frontier text used to seperate different parts of the message
-  my ($bi, @bchrs);
-  my $boundry = "";
-  foreach my $bn (48..57,65..90,97..122) {
-     $bchrs[$bi++] = chr($bn);
+# Checks that all the attachments exist
+  my $self=shift;
+  my $attachs=shift;
+
+  foreach my $attach(@$attachs)
+  {
+     $attach->{file}=~s/\A[\s,\0,\t,\n,\r]*//;
+     $attach->{file}=~s/[\s,\0,\t,\n,\r]*\Z//;
+
+     unless (-f $attach->{file}) {
+       $self->bye;
+       die "Unable to find the attachment file: $attach->{file}\n";
+     }
+     my $opened=open(my $file,'<',$attach->{file});
+     if( not $opened){
+        $self->bye;
+        die "Unable to open the attachment file: $attach->{file}\n";
+     }
+     else{
+       close $file;
+     }
   }
-  foreach my $bn (0..20) {
-     $boundry .= $bchrs[rand($bi)];
-  }
-  return $boundry;
+
+  return 1;
+}
+
+sub _createboundary
+{
+# Create arbitrary frontier text used to separate different parts of the message
+  return "This-is-a-mail-boundary-8217539";
 }
 
 sub send
@@ -130,6 +167,10 @@ sub send
 
   $mail->{to}='';
   $mail->{to}=$properties{'-to'} if defined $properties{'-to'};
+  if($mail->{to} eq ''){
+      print "No RCPT found. Please add the TO field\n";
+      return -1;
+  }
 
   $mail->{from}=$self->{from};
   $mail->{from}=$properties{'-from'} if defined $properties{'-from'};
@@ -158,13 +199,18 @@ sub send
   $mail->{attachments}='';
   $mail->{attachments}=$properties{'-attachments'} if defined $properties{'-attachments'};
 
-  if($self->_checkfiles($mail->{attachments}))
+  $mail->{attachmentlist}=$properties{'-attachmentlist'} if defined $properties{'-attachmentlist'};
+
+  if(($mail->{attachments} ne '')and($self->_checkfiles($mail->{attachments})))
   {
-      print "Attachments successfully verified\n" if $verbose;
+      print "Attachments separated by comma successfully verified\n" if $verbose;
+  }
+  if((defined $mail->{attachmentlist})and($self->_checkfilelist($mail->{attachmentlist}))){
+        print "Attachments \@list successfully verified\n" if $verbose;
   }
 
-  eval{
-      my $boundry=_createboundry();
+  # eval{
+      my $boundary=_createboundary();
 
       $self->{sender}->mail($mail->{from} . "\n");
 
@@ -186,7 +232,7 @@ sub send
       #Send header
       $self->{sender}->datasend("From: " . $mail->{from} . "\n");
       $self->{sender}->datasend("To: " . $mail->{to} . "\n");
-      $self->{sender}->datasend("Cc: " . $mail->{cc} . "\n") if $mail->{cc} ne '';
+      $self->{sender}->datasend("Cc: " . $mail->{cc} . "\n") if ($mail->{cc} ne '');
       $self->{sender}->datasend("Reply-To: " . $mail->{replyto} . "\n");
       $self->{sender}->datasend("Subject: " . $mail->{subject} . "\n");
 
@@ -194,10 +240,10 @@ sub send
       {
         print "With Attachments\n" if $verbose;
         $self->{sender}->datasend("MIME-Version: 1.0\n");
-        $self->{sender}->datasend("Content-Type: multipart/mixed; BOUNDARY=\"$boundry\"\n");
+        $self->{sender}->datasend("Content-Type: multipart/mixed; BOUNDARY=\"$boundary\"\n");
 
         # Send text body
-        $self->{sender}->datasend("\n--$boundry\n");
+        $self->{sender}->datasend("\n--$boundary\n");
         $self->{sender}->datasend("Content-Type: ".$mail->{contenttype}."; charset=".$mail->{charset}."\n");
 
         $self->{sender}->datasend("\n");
@@ -229,39 +275,88 @@ sub send
            my $contentType = guess_media_type($attach);
            print "Composing MIME with attach $attach\n" if $verbose;
            if ($data) {
-              $self->{sender}->datasend("--$boundry\n");
+              $self->{sender}->datasend("--$boundary\n");
               $self->{sender}->datasend("Content-Type: $contentType; name=\"$fileName\"\n");
               $self->{sender}->datasend("Content-Transfer-Encoding: base64\n");
               $self->{sender}->datasend("Content-Disposition: attachment; =filename=\"$fileName\"\n\n");
               $self->{sender}->datasend(encode_base64($data));
-              $self->{sender}->datasend("--$boundry\n");
+              $self->{sender}->datasend("--$boundary\n");
            }
           }
-          $self->{sender}->datasend("\n--$boundry--\n"); # send endboundary end message
+          $self->{sender}->datasend("\n--$boundary--\n"); # send endboundary end message
       }
+
+      elsif(defined $mail->{attachmentlist})
+      {
+        print "With Attachments\n" if $verbose;
+        $self->{sender}->datasend("MIME-Version: 1.0\n");
+        $self->{sender}->datasend("Content-Type: multipart/mixed; BOUNDARY=\"$boundary\"\n");
+
+        # Send text body
+        $self->{sender}->datasend("\n--$boundary\n");
+        $self->{sender}->datasend("Content-Type: ".$mail->{contenttype}."; charset=".$mail->{charset}."\n");
+
+        $self->{sender}->datasend("\n");
+        $self->{sender}->datasend($mail->{body} . "\n\n");
+
+        my $attachments=$mail->{attachmentlist};
+        foreach my $attach(@$attachments)
+        {
+           my($bytesread, $buffer, $data, $total);
+
+           $attach->{file}=~s/\A[\s,\0,\t,\n,\r]*//;
+           $attach->{file}=~s/[\s,\0,\t,\n,\r]*\Z//;
+
+           my $opened=open(my $file,'<',$attach->{file});
+           binmode($file);
+           while (($bytesread = sysread($file, $buffer, 1024)) == 1024) {
+             $total += $bytesread;
+             $data .= $buffer;
+           }
+           if ($bytesread) {
+              $data .= $buffer;
+              $total += $bytesread;
+           }
+           close $file;
+           # Get the file name without its directory
+           my ($volume, $dir, $fileName) = File::Spec->splitpath($attach->{file});
+           # Get the MIME type
+           my $contentType = guess_media_type($attach->{file});
+           print "Composing MIME with attach $attach->{file}\n" if $verbose;
+           if ($data) {
+              $self->{sender}->datasend("--$boundary\n");
+              $self->{sender}->datasend("Content-Type: $contentType; name=\"$fileName\"\n");
+              $self->{sender}->datasend("Content-Transfer-Encoding: base64\n");
+              $self->{sender}->datasend("Content-Disposition: attachment; =filename=\"$fileName\"\n\n");
+              $self->{sender}->datasend(encode_base64($data));
+              $self->{sender}->datasend("--$boundary\n");
+           }
+          }
+          $self->{sender}->datasend("\n--$boundary--\n"); # send endboundary end message
+      }
+
       else { # No attachment
         print "With No attachments\n" if $verbose;
         # Send text body
         $self->{sender}->datasend("MIME-Version: 1.0\n");
         $self->{sender}->datasend("Content-Type: ".$mail->{contenttype}."; charset=".$mail->{charset}."\n");
+
         $self->{sender}->datasend("\n");
         $self->{sender}->datasend($mail->{body}."\n\n");
+
       }
 
       $self->{sender}->datasend("\n");
-      $self->{sender}->dataend();
-      print "Sending email\n" if $verbose;
 
-  }; # eval
+      if($self->{sender}->dataend()) {
+          print "Email sent\n" if $verbose;
+          return 1;
+      }
+      else{
+          print "Sorry, there was an error during sending. Please, retry or use Debug\n" if $verbose;
+          return -1;
+      }
 
-  if($@){
-     print "Warning: $@ \n" if $verbose;
-  }
-  else
-  {
-     print "Mail sent!\n" if $verbose;
-  }
-  return;
 }
 
 1;
@@ -269,7 +364,7 @@ __END__
 
 =head1 NAME
 
-Email::Send::SMTP::Gmail - Sends emails with attachments supporting Auth over TLS or SSL (for example: Google's SMTP). Auth plain and noAuth are also supported
+Email::Send::SMTP::Gmail - Sends emails with attachments supporting Auth over TLS or SSL (for example: Google's SMTP).
 
 =head1 SYNOPSIS
 
@@ -303,12 +398,23 @@ It creates the object and opens a session with the SMTP.
 
 =item I<smtp>: defines SMTP server. Default value: smtp.gmail.com
 
-=item I<layer>: defines the secure layer to use. It could be 'tls', 'ssl', 'plain' or 'none'. Default value: tls
+=item I<layer>: defines the secure layer to use. It could be 'tls', 'ssl' or 'none'. Default value: tls
 
 =item I<port>: defines the port to use. Default values are 25 for tls and 465 for ssl
 
-=item I<debug>: only really works with ssl layer (because currently Net::SMTP::TLS::ButMaintained doesn't support it).
+=item I<auth>: defines the authentication method: ANONYMOUS, CRAM-MD5, DIGEST-MD5, EXTERNAL, GSSAPI, LOGIN (default) and PLAIN. It's currently based on SASL::Perl module
 
+=item I<debug>: see the log information
+
+Also supports SSL parameters as:
+
+=item I<ssl_verify_mode>: SSL_VERIFY_NONE | SSL_VERIFY_PEER
+
+=item I<ssl_verify_path>: SSL_ca_path if SSL_VERIFY_PEER
+
+=item I<ssl_verify_file>: SSL_ca_file if SSL_VERIFY_PEER
+
+=item
 
 =back
 
@@ -324,6 +430,10 @@ It composes and sends the email in one shot
 
 =item I<attachments>: comma separated files with full path
 
+=item I<attachmentslist>: hashref $list, in format $list->[x]->{name} of files with full path. Example: $list->[0]->{file}='/full_path/file.pdf'
+
+=item
+
 =back
 
 =item bye
@@ -349,15 +459,44 @@ Send email composed in HTML using Gmail
       $mail->send(-to=>'target@xxx.com', -subject=>'Hello!', -body=>'Just testing it<br>Bye!',-contenttype=>'text/html');
       $mail->bye;
 
-Send email using a SMTP server without authentication
+Send email using a SMTP server without secure layer and authentication
 
       use strict;
       use warnings;
       use Email::Send::SMTP::Gmail;
-      my $mail=Email::Send::SMTP::Gmail->new( -smtp=>'my.smtp.server', -layer=>'none')
+      my $mail=Email::Send::SMTP::Gmail->new( -smtp=>'my.smtp.server',-layer=>'none', -auth=>'none')
       $mail->send(-from=>'sender@yyy.com', -to=>'target@xxx.com', -subject=>'Hello!', -body=>'Quick email');
       $mail->bye;
 
+Send email with attachments in comma separated format
+
+      use strict;
+      use warnings;
+      use Email::Send::SMTP::Gmail;
+      my $mail=Email::Send::SMTP::Gmail->new( -smtp=>'smtp.gmail.com',
+                                              -login=>'whateveraddress@gmail.com',
+                                              -pass=>'whatever_pass');
+
+      $mail->send(-to=>'target@xxx.com', -subject=>'Hello!', -body=>'Just testing it<br>Bye!',-contenttype=>'text/html',
+                  -attachments=>'/full_path/file1.pdf,/full_path/file2.pdf');
+      $mail->bye;
+
+Send email with attachments using hashref
+
+      use strict;
+      use warnings;
+      use Email::Send::SMTP::Gmail;
+      my $mail=Email::Send::SMTP::Gmail->new( -smtp=>'smtp.gmail.com',
+                                              -login=>'whateveraddress@gmail.com',
+                                              -pass=>'whatever_pass');
+
+      my $att;
+      $att->[0]->{file}='/full_path/file.pdf';
+      $att->[1]->{file}='/full_path/file1.pdf';
+
+      $mail->send(-to=>'target@xxx.com', -subject=>'Hello!', -body=>'Just testing it<br>Bye!',-contenttype=>'text/html', -attachmentlist=>$att);
+      $mail->bye;
+
 =back
 
 =head1 BUGS
